package com.ims.eval.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluationRevision;
import com.ims.eval.entity.dto.response.ResponsibilityIndicatorInfoResDTO;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationRevisionService;
import com.ims.eval.service.IResponsibilityIndicatorInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * 单位考评目标修订
 *
 * @author hlf
 * @date 2023/5/23 9:54
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-revision")
public class EvaluationRevisionController {

	@Autowired
	private IEvaluationRevisionService evaluationRevisionService;

	@Autowired
	private IResponsibilityIndicatorInfoService responsibilityIndicatorInfoService;

	@Autowired
	private HttpServletRequest request;

	/**
	 * 单位考评目标修订列表信息（分页）
	 *
	 * @param pageNum            当前记录起始索引
	 * @param pageSize           每页显示记录数
	 * @param id                 主键
	 * @param responsibilityCode 业务编码
	 * @param cycleUnit          周期单位
	 * @param checkCycle         周期
	 * @param beginDate          考核开始时间
	 * @param endDate            考核截止时间
	 * @param stage              审批状态
	 * @param createBy           创建者
	 * @param year               年
	 * @param month              月
	 * @param des                描述
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(@RequestParam(value = "pageNum") Integer pageNum,
				  @RequestParam(value = "pageSize") Integer pageSize,
				  @RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "responsibilityCode", required = false) String responsibilityCode,
				  @RequestParam(value = "cycleUnit", required = false) String cycleUnit,
				  @RequestParam(value = "checkCycle", required = false) List<String> checkCycle,
				  @RequestParam(value = "beginDate", required = false) String beginDate,
				  @RequestParam(value = "endDate", required = false) String endDate,
				  @RequestParam(value = "stage", required = false) String stage,
				  @RequestParam(value = "createBy", required = false) String createBy,
				  @RequestParam(value = "year", required = false) String year,
				  @RequestParam(value = "month", required = false) String month,
				  @RequestParam(value = "des", required = false) String des) {
		IPage<EvaluationRevision> list = evaluationRevisionService.listPage(pageNum, pageSize, id, responsibilityCode, cycleUnit, checkCycle, beginDate, endDate, stage, createBy, year, month, des);
		return R.ok().data(list);
	}

	/**
	 * 单位考评目标修订信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/listAll")
	public R listAll() {
		List<EvaluationRevision> list = evaluationRevisionService.list();
		return R.ok().data(list);
	}

	/**
	 * 考评修订
	 *
	 * @param id 考评目标主键
	 * @return 结果
	 */
	@GetMapping(value = "/revise")
	public R revise(@RequestParam(value = "id") String id) {
		try {
			boolean b = evaluationRevisionService.save(id);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("考评修订失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 详情
	 *
	 * @param responsibilityId      业务主键
	 * @param indicatorId           indicatorId
	 * @param organizationShortName organizationShortName
	 * @param binSection            binSection
	 * @param binStage              binStage
	 * @return 结果
	 */
	@GetMapping(value = "/details")
	public R details(@RequestParam(value = "responsibilityId", required = false) String responsibilityId,
					 @RequestParam(value = "indicatorId", required = false) String indicatorId,
					 @RequestParam(value = "organizationShortName", required = false) String organizationShortName,
					 @RequestParam(value = "binSection", required = false) String binSection,
					 @RequestParam(value = "binStage", required = false) String binStage) {
		try {
			Map list = responsibilityIndicatorInfoService.getResponsibilityInfoList(responsibilityId, indicatorId, organizationShortName, binSection, binStage, request);
			return R.ok().data(list);
		} catch (CustomException e1) {
			e1.printStackTrace();
			return R.error().customError(e1.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			return R.error().customError("查询失败");
		}
	}

	/**
	 * 详情-批量新增/修改
	 *
	 * @param jsonObjects 修改内容
	 * @return 结果
	 */
	@PostMapping(value = "/save")
	public R addAll(@RequestBody List<JSONObject> jsonObjects) {
		try {
			boolean b = responsibilityIndicatorInfoService.revisionResponsibilityIndicatorInfo(jsonObjects);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("保存失败！");
			}
		} catch (Exception e) {
			log.error("错误", e);
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 详情-弹框数据
	 *
	 * @param id 考评目标主键
	 * @return 结果
	 */
	@GetMapping(value = "/popUpData")
	public R popUpData(@RequestParam(value = "id") String id) {
		List<ResponsibilityIndicatorInfoResDTO> list = responsibilityIndicatorInfoService.selectPopUpData(id, "", "失效");
		return R.ok().data(list);
	}

	/**
	 * 修改状态
	 *
	 * @param id    主键
	 * @param stage 状态
	 * @return 结果
	 */
	@PostMapping(value = "/targetStart")
	public R targetStart(@RequestParam(value = "id") String id,
						 @RequestParam(value = "stage") String stage) {
		try {
			EvaluationRevision evaluationRevision = evaluationRevisionService.getById(id);
			evaluationRevision.setStage(stage);
			boolean b = evaluationRevisionService.updateById(evaluationRevision);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}
}
