package com.ims.eval.controller;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.config.ImaConfig;
import com.ims.eval.entity.EvaluationNotice;
import com.ims.eval.entity.OrganizationStructure;
import com.ims.eval.entity.custom.Path;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationNoticeService;
import com.ims.eval.service.IOrganizationStructureService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;

/**
 * 通告管理
 *
 * @author hlf
 * @date 2023/5/17 14:48
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-dept-notice")
public class NoticeManagementController {

	@Autowired
	private IEvaluationNoticeService evaluationNoticeService;

	@Autowired
	private IOrganizationStructureService organizationStructureService;

	@Autowired
	HttpServletRequest request;

	@Autowired
	private RestTemplate restTemplate;

	@Autowired
	private ImaConfig imaConfig;

	/**
	 * 通告管理列表信息（分页）
	 *
	 * @param pageNum     当前记录起始索引
	 * @param pageSize    每页显示记录数
	 * @param noticeTitle 通告标题
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "noticeTitle", required = false) String noticeTitle) {
		IPage<EvaluationNotice> list = evaluationNoticeService.listPage(pageNum, pageSize, noticeTitle);
		List<EvaluationNotice> evaluationNoticeList = list.getRecords();
		for (EvaluationNotice obj : evaluationNoticeList) {
			StringBuilder names = new StringBuilder();
			if (!"".equals(obj.getSendToContent()) && null != obj.getSendToContent()) {
				String[] strings = obj.getSendToContent().split(",");
				for (String str : strings) {
					List<OrganizationStructure> deptList = organizationStructureService.getTree("23031001", 1, "140");
					if (deptList.size() > 0) {
						OrganizationStructure organizationStructure = deptList.stream().filter(item -> item.getId().equals(str)).findFirst().orElse(null);
						if (ObjectUtil.isNotNull(organizationStructure)) {
							names.append(organizationStructure.getName()).append(",");
						}
					}
				}
				obj.setDeptNames(names.toString().substring(0, names.toString().length() - 1));
			}
		}
		return R.ok().data(list);
	}

	/**
	 * 通告管理信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/listAll")
	public R listAll() {
		List<EvaluationNotice> list = evaluationNoticeService.list();
		return R.ok().data(list);
	}

	/**
	 * 新增/修改通告管理信息
	 *
	 * @param evaluationNotice 通告管理实体
	 * @return 结果
	 */
	@PostMapping(value = "/save")
	public R addAll(@RequestBody EvaluationNotice evaluationNotice) {
		try {
			String id;
			if (StringUtils.isEmpty(evaluationNotice.getId())) {//true：为空
				id = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
				evaluationNotice.setId(id);
				evaluationNotice.setCreateTime(new Date());
			} else {
				id = evaluationNotice.getId();
				evaluationNotice.setUpdateTime(new Date());
			}
			boolean b = evaluationNoticeService.saveOrUpdate(evaluationNotice);
			if (b) {
				return R.ok().data(id);
			} else {
				return R.error("保存失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 批量删除通告管理信息
	 *
	 * @param ids 主键s
	 * @return 结果
	 */
	@PostMapping(value = "/removeAll/{ids}")
	public R deleteAll(@PathVariable("ids") String ids) {
		try {
			String[] strings = ids.split(",");
			boolean b = evaluationNoticeService.removeByIds(Arrays.asList(strings));
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("删除失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 修改状态
	 *
	 * @param ids          多主键
	 * @param releaseState 状态（未发布或已发布或草稿）
	 * @return 结果
	 */
	@PostMapping(value = "/modifiedState")
	public R modifiedState(@RequestParam(value = "ids") String ids,
						   @RequestParam(value = "releaseState") String releaseState) {
		try {
			boolean b = false;
			if (ids.length() > 0) {
				String[] strings = ids.split(",");
				for (String id : strings) {
					EvaluationNotice evaluationNotice = new EvaluationNotice();
					evaluationNotice.setId(id);
					if ("已发布".equals(releaseState)) {
						evaluationNotice.setReleaseDate(new Date());
					} else if ("未发布".equals(releaseState)) {
						evaluationNotice.setReleaseDate(null);
					}
					evaluationNotice.setReleaseState(releaseState);
					b = evaluationNoticeService.updateById(evaluationNotice);
				}
			}
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 上传附件
	 *
	 * @param id    主键
	 * @param files 附件集
	 * @return 结果
	 * @throws IOException 异常
	 */
	@PostMapping(value = "/uploadAttachment")
	public R uploadAttachment(@RequestParam(value = "id") String id, @RequestParam(value = "files", required = false) MultipartFile[] files) throws IOException {
		try {
			boolean b = false;
			EvaluationNotice evaluationNotice = evaluationNoticeService.getById(id);
			if (null != evaluationNotice) {
				for (MultipartFile file : files) {
					HttpHeaders headers = new HttpHeaders();
					headers.setContentType(MediaType.MULTIPART_FORM_DATA);
					headers.add("Blade-Auth", "bearer " + request.getHeader("Blade-Auth"));
					HttpEntity<Map> param = new HttpEntity<>(null, headers);
					ResponseEntity<String> responseEntity2 = restTemplate.exchange(imaConfig.getGatewayUrl() + "f-center/dm/dmDoc/save?folder={1}&objType={2}&objKey={3}&file={4}&bucketName={5}", HttpMethod.POST, param, String.class, "AsApplication", "AsApplication", "", file, "dbkp");
					log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
					if (200 == responseEntity2.getStatusCodeValue()) {

					}
				}
				/*StringBuilder fileNames = new StringBuilder();
				if (files.length > 0) {
					for (MultipartFile file : files) {
						// 上传文件路径
						String filePath = Path.getNoticePath();
						// 上传并返回新文件名称
						String fileName = FileUploadUtil.upload(filePath, file);
						fileNames.append(fileName).append(",");
					}
					String newNoticeAnnex = fileNames.toString().substring(0, fileNames.toString().length() - 1);
					if (StringUtils.isNotEmpty(evaluationNotice.getNoticeAnnex())) {//true：非空串
						String noticeAnnex = evaluationNotice.getNoticeAnnex() + "," + newNoticeAnnex;
						evaluationNotice.setNoticeAnnex(noticeAnnex);
					} else {
						evaluationNotice.setNoticeAnnex(newNoticeAnnex);
					}
					b = evaluationNoticeService.updateById(evaluationNotice);
				}*/
			}
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("上传附件失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 删除附件
	 *
	 * @param id          主键
	 * @param noticeAnnex 附件名称
	 * @return 结果
	 */
	@PostMapping(value = "/deleteAttachment")
	public R deleteAttachment(@RequestParam(value = "id") String id, @RequestParam(value = "noticeAnnex") String noticeAnnex) {
		try {
			boolean b;
			EvaluationNotice evaluationNotice = evaluationNoticeService.getById(id);
			if (null != evaluationNotice) {
				String[] strings = evaluationNotice.getNoticeAnnex().split(",");
				StringBuilder fileNames = new StringBuilder();
				for (String str : strings) {
					if (str.substring(str.lastIndexOf("/") + 1, str.length()).equals(noticeAnnex)) {
						String filePath = Path.getStatementPath() + str;
						File file = new File(filePath);
						b = file.delete();
					} else {
						fileNames.append(str).append(",");
					}
				}
				evaluationNotice.setNoticeAnnex(fileNames.toString().substring(0, fileNames.toString().length() - 1));
			}
			b = evaluationNoticeService.updateById(evaluationNotice);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("删除附件失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 通告展示
	 *
	 * @param deptId      部门主键
	 * @param noticeTitle 通告标题
	 * @return 结果
	 */
	@GetMapping(value = "/circularDisplay")
	public R circularDisplay(@RequestParam(value = "deptId") String deptId,
							 @RequestParam(value = "noticeTitle") String noticeTitle,
							 @RequestParam(value = "beginDataTime", required = false) String beginDataTime,
							 @RequestParam(value = "endDataTime", required = false) String endDataTime) {
		QueryWrapper<EvaluationNotice> qw = new QueryWrapper<>();
		qw.lambda().eq(EvaluationNotice::getReleaseState, "已发布");
		if (StringUtils.isNotEmpty(noticeTitle)) {
			qw.lambda().eq(EvaluationNotice::getNoticeTitle, noticeTitle);
		}
		if (StringUtils.isNotEmpty(beginDataTime) && StringUtils.isNotEmpty(endDataTime)) {
			qw.lambda().between(EvaluationNotice::getReleaseDate, DateUtil.parse(beginDataTime), DateUtil.parse(endDataTime));
		}
		List<EvaluationNotice> evaluationNoticeList = evaluationNoticeService.list(qw);
		List<EvaluationNotice> list = new ArrayList<>();
		for (EvaluationNotice evaluationNotice : evaluationNoticeList) {
			if ("所有单位".equals(evaluationNotice.getSendTo())) {
				if (null != evaluationNotice.getNoticeAnnex()) {
					if (evaluationNotice.getNoticeAnnex().length() > 0) {
						String[] noticeAnnexList = evaluationNotice.getNoticeAnnex().split(",");
						StringBuilder noticeAnnex = new StringBuilder();
						for (String str : noticeAnnexList) {
							if (str.contains("All")) {
								noticeAnnex.append(str).append(",");
							} else if (str.contains(deptId)) {
								noticeAnnex.append(str).append(",");
							}
						}
						if (noticeAnnex.toString().length() > 0) {
							evaluationNotice.setNoticeAnnex(noticeAnnex.toString().substring(0, noticeAnnex.toString().length() - 1));
						}
					}
				}
				list.add(evaluationNotice);
			} else if ("个别部门".equals(evaluationNotice.getSendTo())) {
				String[] sendToContentList = evaluationNotice.getSendToContent().split(",");
				for (String str : sendToContentList) {
					if (str.equals(deptId)) {
						if (evaluationNotice.getNoticeAnnex().length() > 0) {
							String[] noticeAnnexList = evaluationNotice.getNoticeAnnex().split(",");
							StringBuilder noticeAnnex = new StringBuilder();
							for (String str1 : noticeAnnexList) {
								if (str1.contains("All")) {
									noticeAnnex.append(str1).append(",");
								} else if (str1.contains(deptId)) {
									noticeAnnex.append(str1).append(",");
								}
							}
							if (noticeAnnex.toString().length() > 0) {
								evaluationNotice.setNoticeAnnex(noticeAnnex.toString().substring(0, noticeAnnex.toString().length() - 1));
							}
						}
						list.add(evaluationNotice);
						break;
					}
				}
			} else if ("本部部门".equals(evaluationNotice.getSendTo())) {
				List<OrganizationStructure> deptList = organizationStructureService.getTree("23031001", 1, "140");
				if (deptList.size() > 0) {
					OrganizationStructure organizationStructure = deptList.stream().filter(item -> item.getId().equals(deptId)).findFirst().orElse(null);
					if (ObjectUtil.isNotNull(organizationStructure)) {
						list.add(evaluationNotice);
					}
				}
			}
		}
		return R.ok().data(list);
	}

	/**
	 * 文件预览
	 *
	 * @param url 文件路径
	 * @return 结果
	 */
	@GetMapping(value = "/filePreview")
	public R convertToHtml(@RequestParam("url") String url) throws UnknownHostException {
		InetAddress localHost = InetAddress.getLocalHost();
		String ip = localHost.getHostAddress();
		String httpUrl = "http://" + ip + ":28900/file/" + url;
		return R.ok().data(httpUrl);
	}

	static class CommonInputStreamResource extends InputStreamResource {
		private long length;
		private String fileName;

		public CommonInputStreamResource(InputStream inputStream, long length, String fileName) {
			super(inputStream);
			this.length = length;
			this.fileName = fileName;
		}

		/**
		 * 覆写父类方法
		 * 如果不重写这个方法，并且文件有一定大小，那么服务端会出现异常
		 * {@code The multi-part request contained parameter data (excluding uploaded files) that exceeded}
		 */
		@Override
		public String getFilename() {
			return fileName;
		}

		/**
		 * 覆写父类 contentLength 方法
		 * 因为 {@link org.springframework.core.io.AbstractResource#contentLength()}方法会重新读取一遍文件，
		 * 而上传文件时，restTemplate 会通过这个方法获取大小。然后当真正需要读取内容的时候，发现已经读完，会报如下错误。
		 */
		@Override
		public long contentLength() {
			long estimate = length;
			return estimate == 0 ? 1 : estimate;
		}

		public void setLength(long length) {
			this.length = length;
		}

		public void setFileName(String fileName) {
			this.fileName = fileName;
		}
	}
}
